import os
import numpy as np
import random
import pickle
import zipfile
from easydict import EasyDict
from scipy.io import loadmat

np.random.seed(0)
random.seed(0)

group_order = [7, 8, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 9, 10, 11, 12, 1, 2, 3, 0, 4, 5, 6]


def make_dir(path):
    if os.path.exists(path):
        pass
    else:
        os.mkdir(path)


def generate_data_description(save_dir, reorder):
    """
    create a dataset description file(pkl file), which consists of images, labels
    """
    with zipfile.ZipFile(r'\Users\licha\Desktop\行人分析数据集\PA100K\annotation.zip', 'r') as zfile:
        with zfile.open('annotation.mat') as mat:
            pa100k_data = loadmat(mat)

    dataset = EasyDict()
    dataset.description = 'pa100k'
    dataset.reorder = 'group_order'
    dataset.root = os.path.join(save_dir, 'data')

    train_image_name = [pa100k_data['train_images_name'][i][0][0] for i in range(80000)]
    val_image_name = [pa100k_data['val_images_name'][i][0][0] for i in range(10000)]
    test_image_name = [pa100k_data['test_images_name'][i][0][0] for i in range(10000)]
    dataset.image_name = train_image_name + val_image_name + test_image_name

    dataset.label = np.concatenate((pa100k_data['train_label'], pa100k_data['val_label'], pa100k_data['test_label']), axis=0)
    dataset.attr_name = [pa100k_data['attributes'][i][0][0] for i in range(26)]

    dataset.label_idx = EasyDict()
    dataset.label_idx.eval = list(range(26))

    if reorder:
        dataset.label_idx.eval = group_order

    dataset.partition = EasyDict()
    dataset.partition.train = np.arange(0, 80000)  # np.array(range(80000))
    dataset.partition.val = np.arange(80000, 90000)  # np.array(range(80000, 90000))
    dataset.partition.test = np.arange(90000, 100000)  # np.array(range(90000, 100000))
    dataset.partition.trainval = np.arange(0, 90000)  # np.array(range(90000))

    dataset.weight_train = np.mean(dataset.label[dataset.partition.train], axis=0).astype(np.float32)
    dataset.weight_trainval = np.mean(dataset.label[dataset.partition.trainval], axis=0).astype(np.float32)

    with open(os.path.join(r'C:\Users\licha\PycharmProjects\attribute-based_retrieval\data\PA100K', 'dataset_all.pkl'), 'wb+') as f:
        pickle.dump(dataset, f)


if __name__ == "__main__":
    save_dir = r'C:\Users\licha\Desktop\行人分析数据集\PA100K'
    # save_dir = './data/PA100k/'
    reoder = True
    generate_data_description(save_dir, reorder=True)